@@ -20,3 +20,7 @@
 
 /* Define on non Unix OS'es which don't have the concept of tty's */
 #undef USE_RAWTERM
+
+
+/* Define when sys_errlist is defined in the standard include files */
+#undef DECL_SYS_ERRLIST
