@@ -1,7 +1,7 @@
 #include "sysincludes.h"
 #include "mtools.h"
 #include "codepage.h"
-#include "paths.h"
+#include "mtoolsPaths.h"
 
 /* global variables */
 /* they are not really harmful here, because there is only one configuration
@@ -21,6 +21,9 @@ static const char *filename; /* current file name. Only used for printing
 			      * error messages */
 static int file_nr=0;
 
+
+static int flag_mask; /* mask of currently set flags */
+
 /* devices */
 static int cur_devs; /* current number of defined devices */
 static int cur_dev; /* device being filled in. If negative, none */
@@ -67,10 +70,12 @@ static switches_t switches[] = {
     { "COUNTRY", (caddr_t) &country_string, STRING }
 };
 
-static struct {
+typedef struct {
     const char *name;
     int flag;
-} openflags[] = {
+} flags_t;
+
+static flags_t openflags[] = {
 #ifdef O_SYNC
     { "sync",		O_SYNC },
 #endif
@@ -82,6 +87,17 @@ static struct {
 #endif
 };
 
+static flags_t misc_flags[] = {
+#ifdef USE_XDF
+	{ "use_xdf",		USE_XDF_FLAG },
+#endif
+	{ "scsi",			SCSI_FLAG },
+	{ "nolock",			NOLOCK_FLAG },
+	{ "mformat_only",	MFORMAT_ONLY_FLAG },
+	{ "privileged",		PRIV_FLAG },
+	{ "vold",			VOLD_FLAG }
+};
+
 static struct {
     const char *name;
     signed char fat_bits;
@@ -116,10 +132,6 @@ static switches_t dswitches[]= {
     { "FILE", OFFS(name), STRING },
     { "OFFSET", OFFS(offset), INT },
     { "PARTITION", OFFS(partition), INT },
-    { "SCSI", OFFS(scsi), INT },
-    { "PRIVILEGED", OFFS(privileged), INT },
-    { "VOLD", OFFS(vold), INT },
-    { "NOLOCK", OFFS(nolock), INT },
     { "FAT", OFFS(fat_bits), INT },
     { "FAT_BITS", OFFS(fat_bits), INT },
     { "MODE", OFFS(mode), INT },
@@ -127,9 +139,9 @@ static switches_t dswitches[]= {
     { "CYLINDERS",  OFFS(tracks), INT },
     { "HEADS", OFFS(heads), INT },
     { "SECTORS", OFFS(sectors), INT },
-    { "USE_XDF", OFFS(use_xdf), INT },
-    { "PRECMD", OFFS(precmd), STRING }
-
+    { "HIDDEN", OFFS(hidden), INT },
+    { "PRECMD", OFFS(precmd), STRING },
+    { "BLOCKSIZE", OFFS(blocksize), INT }
 };
 
 static void syntax(const char *msg)
@@ -267,7 +279,7 @@ static void grow(void)
 	if(cur_devs >= nr_dev - 2) {
 		nr_dev = (cur_devs + 2) << 1;
 		if(!(devices=Grow(devices, nr_dev, struct device))){
-			fprintf(stderr,"Out of memory error");
+			printOom();
 			exit(1);
 		}
 	}
@@ -320,10 +332,10 @@ static void finish_drive_clause(void)
 	    !devices[cur_dev].sectors))
 		syntax("incomplete geometry: either indicate all of track/heads/sectors or none of them");
 	devices[cur_dev].file_nr = file_nr;
-	if(devices[cur_dev].privileged == 2)
-		devices[cur_dev].privileged = devices[cur_dev].scsi;
+	if(! (flag_mask & PRIV_FLAG) && IS_SCSI(&devices[cur_dev]))
+		devices[cur_dev].misc_flags |= PRIV_FLAG;
 	if(!trusted)
-		devices[cur_dev].privileged = 0;
+		devices[cur_dev].misc_flags &= ~PRIV_FLAG;
 	trusted = 0;
 	cur_dev = -1;
 }
@@ -360,6 +372,32 @@ static int set_openflags(struct device *dev)
 	return 1;
 }
 
+static int set_misc_flags(struct device *dev)
+{
+	int i;
+
+	for(i=0; i < sizeof(misc_flags) / sizeof(*misc_flags); i++) {
+		if(match_token(misc_flags[i].name)) {
+			flag_mask |= misc_flags[i].flag;
+			skip_junk(0);
+			if(pos && *pos == '=') {
+				pos++;
+				switch(get_number()) {
+					case 0:
+						return 0;
+					case 1:
+						break;
+					default:
+						syntax("expected 0 or 1");
+				}
+			}
+			dev->misc_flags |= misc_flags[i].flag;
+			return 0;
+		}
+	}
+	return 1;
+}
+
 static int set_def_format(struct device *dev)
 {
 	int i;
@@ -402,7 +440,7 @@ static void get_toupper(void)
 	int i;
 
 	if(!mstoupper)
-		mstoupper = malloc(128);
+		mstoupper = safe_malloc(128);
 	for(i=0; i<128; i++)
 		mstoupper[i] = get_number();
 }
@@ -454,7 +492,7 @@ static void parse_old_device_line(char drive)
 	
 	devices[cur_dev].drive = toupper(devices[cur_dev].drive);
 	if (!(devices[cur_dev].name = strdup(name))) {
-		fprintf(stderr,"Out of memory\n");
+		printOom();
 		exit(1);
 	}
 	finish_drive_clause();
@@ -491,7 +529,7 @@ static int parse_one(int privilege)
 			append();
 		memset((char*)(devices+cur_dev), 0, sizeof(*devices));
 		trusted = privilege;
-		devices[cur_dev].privileged = 2;
+		flag_mask = 0;
 		devices[cur_dev].drive = toupper(token[0]);
 		expect_char(':');
 		return 1;
@@ -524,6 +562,7 @@ static int parse_one(int privilege)
 		     sizeof(dswitches)/sizeof(*dswitches),
 		     (caddr_t)&devices[cur_dev]) &&
 	     set_openflags(&devices[cur_dev]) &&
+	     set_misc_flags(&devices[cur_dev]) &&
 	     set_def_format(&devices[cur_dev]))) &&
 	   set_var(switches,
 		   sizeof(switches)/sizeof(*switches), 0))
@@ -562,7 +601,7 @@ void read_config(void)
 	nr_dev = nr_const_devices + 2;
 	devices = NewArray(nr_dev, struct device);
 	if(!devices) {
-		fprintf(stderr,"Out of memory error\n");
+		printOom();
 		exit(1);
 	}
 	if(nr_const_devices)
@@ -602,24 +641,43 @@ void mtoolstest(int argc, char **argv, int type)
 	/* testing purposes only */
 	struct device *dev;
 	int i,j;
+	char drive='\0';
+
+	if(argc > 1 && argv[1][0] && argv[1][1] == ':') {
+		drive = toupper(argv[1][0]);
+	}
+
 	for (dev=devices; dev->name; dev++) {
+		if(drive && drive != dev->drive)
+			continue;
 		printf("drive %c:\n", dev->drive);
 		printf("\t#fn=%d mode=%d\n",
 		       dev->file_nr, dev->mode);
 		printf("\tfile=\"%s\" fat_bits=%d \n",
 		       dev->name,dev->fat_bits);
-		printf("\ttracks=%d heads=%d sectors=%d\n",
-		       dev->tracks, dev->heads, dev->sectors);
-		printf("\tvold=%lx\n", (long) dev->vold);
+		printf("\ttracks=%d heads=%d sectors=%d hidden=%d\n",
+		       dev->tracks, dev->heads, dev->sectors, dev->hidden);
 		printf("\toffset=0x%lx\n", (long) dev->offset);
-		printf("\tpartition=0x%x\n", dev->partition);
-		printf("\tscsi=0x%x\n", dev->scsi);
-		printf("\tnolock=0x%x\n", dev->nolock);
-		printf("\tprivileged=0x%x\n", dev->privileged);
+		printf("\tpartition=%d\n", dev->partition);
 
+		if(dev->misc_flags)
+			printf("\t");
+
+		if(IS_SCSI(dev))
+			printf("scsi ");
+		if(IS_PRIVILEGED(dev))
+			printf("privileged");
+		if(IS_MFORMAT_ONLY(dev))
+			printf("mformat_only ");
+		if(SHOULD_USE_VOLD(dev))
+			printf("vold ");
 #ifdef USE_XDF
-		printf("\tuse_xdf=0x%x\n", dev->use_xdf);
+		if(SHOULD_USE_XDF(dev))
+			printf("use_xdf ");
 #endif
+		if(dev->misc_flags)
+			printf("\n");
+
 		if(dev->mode)
 			printf("\t");
 #ifdef O_SYNC
