@@ -13,31 +13,51 @@
 #include "mtools.h"
 #include "devices.h"
 
-#define DEF_ARG1 0, 0, 0x2,0,0, 0, 0, 0
-#define DEF_ARG0 0,0,0,DEF_ARG1
-#define DEF_ARG 0L,DEF_ARG0
+#define INIT_NOOP
 
-#define VOLD_DEF_ARG1 1, 0, 0x2,0,0, 0, 0, 0
-#define VOLD_DEF_ARG0 0,0,0,DEF_ARG1
-#define VOLD_DEF_ARG 0L,DEF_ARG0
+#define DEF_ARG1(x) (x), 0x2,0,(char *)0, 0, 0
+#define DEF_ARG0(x) 0,DEF_ARG1(x)
+
+#define MDEF_ARG 0L,DEF_ARG0(MFORMAT_ONLY_FLAG)
+#define FDEF_ARG 0L,DEF_ARG0(0)
+#define VOLD_DEF_ARG 0L,DEF_ARG0(VOLD_FLAG)
+
+#define MED312	12,0,80,2,36,0,MDEF_ARG /* 3 1/2 extra density */
+#define MHD312	12,0,80,2,18,0,MDEF_ARG /* 3 1/2 high density */
+#define MDD312	12,0,80,2, 9,0,MDEF_ARG /* 3 1/2 double density */
+#define MHD514	12,0,80,2,15,0,MDEF_ARG /* 5 1/4 high density */
+#define MDD514	12,0,40,2, 9,0,MDEF_ARG /* 5 1/4 double density (360k) */
+#define MSS514	12,0,40,1, 9,0,MDEF_ARG /* 5 1/4 single sided DD, (180k) */
+#define MDDsmall	12,0,40,2, 8,0,MDEF_ARG /* 5 1/4 double density (320k) */
+#define MSSsmall	12,0,40,1, 8,0,MDEF_ARG /* 5 1/4 single sided DD, (160k) */
+
+#define FED312	12,0,80,2,36,0,FDEF_ARG /* 3 1/2 extra density */
+#define FHD312	12,0,80,2,18,0,FDEF_ARG /* 3 1/2 high density */
+#define FDD312	12,0,80,2, 9,0,FDEF_ARG /* 3 1/2 double density */
+#define FHD514	12,0,80,2,15,0,FDEF_ARG /* 5 1/4 high density */
+#define FDD514	12,0,40,2, 9,0,FDEF_ARG /* 5 1/4 double density (360k) */
+#define FSS514	12,0,40,1, 9,0,FDEF_ARG /* 5 1/4 single sided DD, (180k) */
+#define FDDsmall	12,0,40,2, 8,0,FDEF_ARG /* 5 1/4 double density (320k) */
+#define FSSsmall	12,0,40,1, 8,0,FDEF_ARG /* 5 1/4 single sided DD, (160k) */
+
+#define GENHD	16,0, 0,0, 0,0,MDEF_ARG /* Generic 16 bit FAT fs */
+#define GENFD	12,0,80,2,18,0,MDEF_ARG /* Generic 12 bit FAT fs */
+#define VOLDFD	12,0, 0,0, 0,0,VOLD_DEF_ARG /* Generic 12 bit FAT fs with vold */
+#define GEN    	 0,0, 0,0, 0,0,MDEF_ARG /* Generic fs of any FAT bits */
+
+#define ZIPJAZ(x,c,h,s,y) 16,(x),(c),(h),(s),(s),0L, 4, \
+		DEF_ARG1((y)|MFORMAT_ONLY_FLAG) /* Jaz disks */
+
+#define JAZ(x)	 ZIPJAZ(x,1021, 64, 32, 0)
+#define RJAZ(x)	 ZIPJAZ(x,1021, 64, 32, SCSI_FLAG|PRIV_FLAG)
+#define ZIP(x)	 ZIPJAZ(x,96, 64, 32, 0)
+#define RZIP(x)	 ZIPJAZ(x,96, 64, 32, SCSI_FLAG|PRIV_FLAG)
 
-#define ED312	12,0,80,2,36,DEF_ARG /* 3 1/2 extra density */
-#define HD312	12,0,80,2,18,DEF_ARG /* 3 1/2 high density */
-#define DD312	12,0,80,2, 9,DEF_ARG /* 3 1/2 double density */
-#define HD514	12,0,80,2,15,DEF_ARG /* 5 1/4 high density */
-#define DD514	12,0,40,2, 9,DEF_ARG /* 5 1/4 double density (360k) */
-#define DDsmall	12,0,40,2, 8,DEF_ARG /* 5 1/4 double density (320k) */
-#define SS514	12,0,40,1, 9,DEF_ARG /* 5 1/4 single sided DD, (180k) */
-#define SSsmall	12,0,40,1, 8,DEF_ARG /* 5 1/4 single sided DD, (160k) */
-#define GENHD	16,0, 0,0, 0,DEF_ARG /* Generic 16 bit FAT fs */
-#define GENFD	12,0, 0,0, 0,DEF_ARG /* Generic 12 bit FAT fs */
-#define VOLDFD	12,0, 0,0, 0,VOLD_DEF_ARG /* Generic 12 bit FAT fs with vold */
-#define GEN    	 0,0, 0,0, 0,DEF_ARG /* Generic fs of any FAT bits */
-#define ZIPJAZ(x)	 0,x, 0,0, 0,0L,4,0,0,DEF_ARG1 /* Zip or Jaz disks */
-#define RZIPJAZ(x)	 0,x, 0,0, 0,0L,4,1,1,DEF_ARG1 /* Zip or Jaz disks */
 
 static int compare_geom(struct device *dev, struct device *orig_dev)
 {
+	if(IS_MFORMAT_ONLY(orig_dev))
+		return 0; /* geometry only for mformatting ==> ok */
 	if(!orig_dev || !orig_dev->tracks || !dev || !dev->tracks)
 		return 0; /* no original device. This is ok */
 	return(orig_dev->tracks != dev->tracks ||
@@ -52,61 +72,60 @@ static int compare_geom(struct device *dev, struct device *orig_dev)
 #define predefined_devices
 struct device devices[] = {
    {"/dev/floppy0", 'A', GENFD },
-   {"/dev/rdsk/c104d0s31", 'J', ZIPJAZ(O_EXCL) },
-   {"/dev/rdsk/c105d0s31", 'Z', ZIPJAZ(O_EXCL) }
+   {"/dev/rdsk/c104d0s31", 'J', JAZ(O_EXCL) },
+   {"/dev/rdsk/c105d0s31", 'Z', ZIP(O_EXCL) }
 };
-#define INIT_NOOP
 #endif /* aux */
 
 
 #ifdef OS_lynxos
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/fd1440.0", 	'A', HD312 },
+	{"/dev/fd1440.0", 	'A', MHD312 },
 };
-#define INIT_NOOP
 #endif
 
 
 #ifdef __BEOS__
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/disk/floppy/raw", 	'A', HD312 },
+	{"/dev/disk/floppy/raw", 	'A', MHD312 },
 };
-#define INIT_NOOP
 #endif /* BEBOX */
 
 
 #ifdef OS_hpux
-#include <sys/floppy.h>
-#undef SSIZE
 
 #define predefined_devices
 struct device devices[] = {
 #ifdef OS_hpux10
 /* hpux10 uses different device names according to Frank Maritato
  * <frank@math.hmc.edu> */
-	{"/dev/floppy/c0t0d0",		'A', HD312 },
-	{"/dev/floppy/c0t0d1",		'B', HD312 }, /* guessed by me */
+	{"/dev/floppy/c0t0d0",		'A', MHD312 },
+	{"/dev/floppy/c0t0d1",		'B', MHD312 }, /* guessed by me */
  	{"/dev/rscsi",			'C', GENHD }, /* guessed by me */
 #else
 /* Use rfloppy, according to Simao Campos <simao@iris.ctd.comsat.com> */
-	{"/dev/rfloppy/c201d0s0",	'A', HD312 },
-	{"/dev/rfloppy/c20Ad0s0", 	'A', HD312 },
- 	{"/dev/rfloppy/c201d1s0",	'B', HD312 },
- 	{"/dev/rfloppy/c20Ad1s0",	'B', HD312 },
+	{"/dev/rfloppy/c201d0s0",	'A', FHD312 },
+	{"/dev/rfloppy/c20Ad0s0", 	'A', FHD312 },
+ 	{"/dev/rfloppy/c201d1s0",	'B', FHD312 },
+ 	{"/dev/rfloppy/c20Ad1s0",	'B', FHD312 },
  	{"/dev/rscsi",			'C', GENHD },
 #endif
-	{"/dev/rdsk/c201d4",		'J', ZIPJAZ(O_EXCL) },
-	{"/dev/rdsk/c201d4s0",		'J', ZIPJAZ(O_EXCL) },
-	{"/dev/rdsk/c201d5",		'Z', ZIPJAZ(O_EXCL) },
-	{"/dev/rdsk/c201d5s0",		'Z', ZIPJAZ(O_EXCL) },
+	{"/dev/rdsk/c201d4",		'J', JAZ(O_EXCL) },
+	{"/dev/rdsk/c201d4s0",		'J', JAZ(O_EXCL) },
+	{"/dev/rdsk/c201d5",		'Z', ZIP(O_EXCL) },
+	{"/dev/rdsk/c201d5s0",		'Z', ZIP(O_EXCL) },
 };
-#ifdef OS_hpux9
+
+#ifdef HAVE_SYS_FLOPPY
 /* geometry setting ioctl's contributed by Paolo Zeppegno
  * <paolo@to.sem.it>, may cause "Not a typewriter" messages on other
  * versions according to support@vital.com */
 
+#include <sys/floppy.h>
+#undef SSIZE
+
 struct generic_floppy_struct
 {
   struct floppy_geometry fg;
@@ -142,8 +161,6 @@ static inline int set_parameters(int fd, struct generic_floppy_struct *floppy,
 	return 0;
 }
 #define INIT_GENERIC
-#else
-#define INIT_NOOP
 #endif
 
 #endif /* hpux */
@@ -153,20 +170,19 @@ static inline int set_parameters(int fd, struct generic_floppy_struct *floppy,
 #define predefined_devices
 struct device devices[] = {
 #ifdef CPU_mips     /* for Siemens Nixdorf's  SINIX-N/O (mips) 5.4x SVR4 */
-	{ "/dev/at/flp/f0t",    'A', HD312},
-	{ "/dev/fd0",           'A', GENFD},
+	{ "/dev/at/flp/f0t",    'A', FHD312},
+	{ "/dev/fd0",           'A', FGENFD},
 #else
 #ifdef CPU_i386     /* for Siemens Nixdorf's  SINIX-D/L (intel) 5.4x SVR4 */
-	{ "/dev/fd0135ds18",	'A', HD312},
-	{ "/dev/fd0135ds9",	'A', DD312},
+	{ "/dev/fd0135ds18",	'A', FHD312},
+	{ "/dev/fd0135ds9",	'A', FDD312},
 	{ "/dev/fd0",		'A', GENFD},
-	{ "/dev/fd1135ds15",	'B', HD514},
-	{ "/dev/fd1135ds9",	'B', DD514},
+	{ "/dev/fd1135ds15",	'B', FHD514},
+	{ "/dev/fd1135ds9",	'B', FDD514},
 	{ "/dev/fd1",		'B', GENFD},
 #endif /* CPU_i386 */
 #endif /*mips*/
 };
-#define INIT_NOOP
 #endif
 
 #ifdef OS_ultrix
@@ -176,7 +192,6 @@ struct device devices[] = {
 	{"/dev/rfd0c",		'A', GENFD}, /* guessed */
 };
 
-#define INIT_NOOP
 #endif
 
 
@@ -184,16 +199,15 @@ struct device devices[] = {
 #define predefined_devices
 #if (defined(OS_isc2) && defined(OLDSTUFF))
 struct device devices[] = {
-	{"/dev/rdsk/f0d9dt",   	'A', DD514},
-	{"/dev/rdsk/f0q15dt",	'A', HD514},
-	{"/dev/rdsk/f0d8dt",	'A', DDsmall},
-	{"/dev/rdsk/f13ht",	'B', HD312},
-	{"/dev/rdsk/f13dt",	'B', DD312},
+	{"/dev/rdsk/f0d9dt",   	'A', FDD514},
+	{"/dev/rdsk/f0q15dt",	'A', FHD514},
+	{"/dev/rdsk/f0d8dt",	'A', FDDsmall},
+	{"/dev/rdsk/f13ht",	'B', FHD312},
+	{"/dev/rdsk/f13dt",	'B', FDD312},
 	{"/dev/rdsk/0p1",	'C', GENHD},
 	{"/usr/vpix/defaults/C:",'D',12, 0, 0, 0, 0,8704L,DEF_ARG0},
 	{"$HOME/vpix/C:", 	'E', 12, 0, 0, 0, 0,8704L,DEF_ARG},
 };
-#define INIT_NOOP
 #else
 /* contributed by larry.jones@sdrc.com (Larry Jones) */
 struct device devices[] = {
@@ -218,10 +232,17 @@ static int get_parameters(int fd, struct generic_floppy_struct *floppy)
 	char buf[512];
 
 	off = lseek(fd, 0, SEEK_CUR);
+	if(off < 0) {
+		perror("device seek 1");
+		exit(1);
+	}
 	if (off == 0) {
 		/* need to read at least 1 sector to get correct info */
 		read(fd, buf, sizeof buf);
-		lseek(fd, 0, SEEK_SET);
+		if(lseek(fd, 0, SEEK_SET) < 0) {
+			perror("device seek 2");
+			exit(1);
+		}
 	}
 	return ioctl(fd, V_GETPARMS, floppy);
 }
@@ -247,7 +268,6 @@ static int set_parameters(int fd, struct generic_floppy_struct *floppy,
 struct device devices[] = {
 	{"/dev/rfd0", 'A', GENFD},
 };
-#define INIT_NOOP
 #endif /* CPU_i370 */
 
 #ifdef OS_aix
@@ -256,7 +276,6 @@ struct device devices[] = {
 struct device devices[] = {
   {"/dev/fd0",'A',GENFD},
 };
-#define INIT_NOOP
 #endif /* aix */
 
   
@@ -266,7 +285,6 @@ struct device devices[] = {
 struct device devices[] = {
   {"/dev/fd0c",'A',GENFD},
 };
-#define INIT_NOOP
 #endif /* OS_osf4 */
 
 
@@ -283,9 +301,9 @@ char *getVoldName(struct device *dev, char *name)
 {
 	char *rname;
   
-	if(!dev->vold)
+	if(!IS_VOLD(dev))
 		return name;
-	
+
 	/***
 	 * Solaris specific routines to use the volume management
 	 * daemon and libraries to get the correct device name...
@@ -316,8 +334,8 @@ struct device devices[] = {
 	{"/dev/rdiskette", 'A', GENFD},
 	{"/vol/dev/aliases/floppy0", 'B', GENFD},
 #endif	/* USING_VOLD */
-	{"/dev/rdsk/c0t4d0s2", 'J', RZIPJAZ(O_NDELAY)},
-	{"/dev/rdsk/c0t5d0s2", 'Z', RZIPJAZ(O_NDELAY)},
+	{"/dev/rdsk/c0t4d0s2", 'J', RJAZ(O_NDELAY)},
+	{"/dev/rdsk/c0t5d0s2", 'Z', RZIP(O_NDELAY)},
 };
 
 
@@ -372,36 +390,33 @@ static inline int set_parameters(int fd, struct generic_floppy_struct *floppy,
 #ifdef OS_sunos3
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/rfdl0c",	'A', DD312},
-	{"/dev/rfd0c",	'A', HD312},
+	{"/dev/rfdl0c",	'A', FDD312},
+	{"/dev/rfd0c",	'A', FHD312},
 };
-#define INIT_NOOP
 #endif /* OS_sunos3 */
 
 #ifdef OS_xenix
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/fd096ds15",	'A', HD514},
-	{"/dev/fd048ds9",	'A', DD514},
-	{"/dev/fd1135ds18",	'B', HD312},
-	{"/dev/fd1135ds9",	'B', DD312},
+	{"/dev/fd096ds15",	'A', FHD514},
+	{"/dev/fd048ds9",	'A', FDD514},
+	{"/dev/fd1135ds18",	'B', FHD312},
+	{"/dev/fd1135ds9",	'B', FDD312},
 	{"/dev/hd0d",		'C', GENHD},
 };
-#define INIT_NOOP
 #endif /* OS_xenix */
 
 #ifdef OS_sco
 #define predefined_devices
 struct device devices[] = {
-	{ "/dev/fd0135ds18",	'A', HD312},
-	{ "/dev/fd0135ds9",	'A', DD312},
+	{ "/dev/fd0135ds18",	'A', FHD312},
+	{ "/dev/fd0135ds9",	'A', FDD312},
 	{ "/dev/fd0",		'A', GENFD},
-	{ "/dev/fd1135ds15",	'B', HD514},
-	{ "/dev/fd1135ds9",	'B', DD514},
+	{ "/dev/fd1135ds15",	'B', FHD514},
+	{ "/dev/fd1135ds9",	'B', FDD514},
 	{ "/dev/fd1",		'B', GENFD},
 	{ "/dev/hd0d",		'C', GENHD},
 };
-#define INIT_NOOP
 #endif /* OS_sco */
 
 
@@ -413,7 +428,6 @@ struct device devices[] = {
   { "/dev/rdsk/fds0d2.96",	'A', HD514},
   {"/dev/rdsk/fds0d2.48",	'A', DD514},
 };
-#define INIT_NOOP
 #endif /* OS_irix */
 
 
@@ -424,8 +438,8 @@ struct device devices[] = {
 #define predefined_devices
 struct device devices[] = {
 	{"/dev/rfd0c",	'A', GENFD},
-	{"/dev/rsd4c",	'J', RZIPJAZ(O_NDELAY)},
-	{"/dev/rsd5c",	'Z', RZIPJAZ(O_NDELAY)},
+	{"/dev/rsd4c",	'J', RJAZ(O_NDELAY)},
+	{"/dev/rsd5c",	'Z', RZIP(O_NDELAY)},
 };
 
 /*
@@ -493,26 +507,24 @@ static inline int set_parameters(int fd, struct generic_floppy_struct *floppy,
 #define predefined_devices
 struct device devices[] = {
 	/* [block device]: DPX1000 has /dev/flbm60, DPX2 has /dev/easyfb */
-	{"/dev/flbm60", 'A', HD514};
-	{"/dev/flbm60", 'B', DD514},
-	{"/dev/flbm60", 'C', DDsmall},
-	{"/dev/flbm60", 'D', SS},
-	{"/dev/flbm60", 'E', SSsmall},
+	{"/dev/flbm60", 'A', MHD514};
+	{"/dev/flbm60", 'B', MDD514},
+	{"/dev/flbm60", 'C', MDDsmall},
+	{"/dev/flbm60", 'D', MSS},
+	{"/dev/flbm60", 'E', MSSsmall},
 };
-#define INIT_NOOP
 #endif /* DPX1000 */
 
 #ifdef OS_bosx
 #define predefined_devices
 struct device devices[] = {
 	/* [block device]: DPX1000 has /dev/flbm60, DPX2 has /dev/easyfb */
-	{"/dev/easyfb", 'A', HD514},
-	{"/dev/easyfb", 'B', DD514},
-	{"/dev/easyfb", 'C', DDsmall},
-	{"/dev/easyfb", 'D', SS},
-	{"/dev/easyfb", 'E', SSsmall},
+	{"/dev/easyfb", 'A', MHD514},
+	{"/dev/easyfb", 'B', MDD514},
+	{"/dev/easyfb", 'C', MDDsmall},
+	{"/dev/easyfb", 'D', MSS},
+	{"/dev/easyfb", 'E', MSSsmall},
 };
-#define INIT_NOOP
 #endif /* OS_bosx */
 
 #ifdef OS_linux
@@ -662,8 +674,8 @@ int analyze_one_reply(RawRequest_t *raw_cmd, int *bytes, int do_print)
 
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/fd0", 'A', 0, O_EXCL, 0,0, 0,DEF_ARG},
-	{"/dev/fd1", 'B', 0, O_EXCL, 0,0, 0,DEF_ARG},
+	{"/dev/fd0", 'A', 0, O_EXCL, 0,0, 0,0, FDEF_ARG},
+	{"/dev/fd1", 'B', 0, O_EXCL, 0,0, 0,0, FDEF_ARG},
 	/* we assume that the Zip or Jaz drive is the second on the SCSI bus */
 	{"/dev/sdb4",'J', GENHD },
 	{"/dev/sdb4",'Z', GENHD },
@@ -729,27 +741,25 @@ static inline int get_parameters(int fd, struct floppy_struct *floppy)
 #ifdef OS_freebsd
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/rfd0.1440", 'A', HD312},
-	{"/dev/rfd0.720",  'A', DD312},
-	{"/dev/rfd1.1200", 'B', HD514},
+	{"/dev/rfd0.1440", 'A', FHD312},
+	{"/dev/rfd0.720",  'A', FDD312},
+	{"/dev/rfd1.1200", 'B', MHD514},
 	{"/dev/sd0s1",     'C', GENHD},
 };
-#define INIT_NOOP
 #endif /* __FreeBSD__ */
  
 /*** /jes -- for ALR 486 DX4/100 ***/
 #ifdef OS_netbsd
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/rfd0a", 'A', HD312},
-	{"/dev/rfd0f", 'A', DD312},
-	{"/dev/rfd0f", 'S', DD312},
-	{"/dev/rfd1a", 'B', HD514},
-	{"/dev/rfd1d", 'B', DD514},
-	{"/dev/rfd1d", 'T', DD514},
-	{"/dev/rwd0d", 'C', 16, 0, 0, 0, 0, 63L*512L, DEF_ARG0},
+	{"/dev/rfd0a", 'A', FHD312},
+	{"/dev/rfd0f", 'A', FDD312},
+	{"/dev/rfd0f", 'S', MDD312},
+	{"/dev/rfd1a", 'B', FHD514},
+	{"/dev/rfd1d", 'B', FDD514},
+	{"/dev/rfd1d", 'T', MDD514},
+	{"/dev/rwd0d", 'C', 16, 0, 0, 0, 0, 0, 63L*512L, DEF_ARG0(0)},
 };
-#define INIT_NOOP
 #endif /* OS_NetBSD */
 
 
@@ -758,10 +768,11 @@ struct device devices[] = {
 
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/rfp020",		'A', 12,O_NDELAY,40,2, 9,DEF_ARG},
+	{"/dev/rfp020",		'A', 12,O_NDELAY,40,2, 9, 0, DEF_ARG},
 	{"/usr/bin/DOS/dvd000", 'C', GENFD},
 };
 
+#undef INIT_NOOP
 int init_geom(int fd, struct device *dev, struct device *orig_dev,
 	      struct stat *stat)
 {
@@ -804,7 +815,6 @@ int init_geom(int fd, struct device *dev, struct device *orig_dev,
 struct device devices[] = {
 	{"/dev/rfd0c",		'A', GENFD},
 };
-#define INIT_NOOP
 #endif
 #endif
 #endif
@@ -813,8 +823,7 @@ struct device devices[] = {
 #ifndef predefined_devices
 #define predefined_devices
 struct device devices[] = {
-  {"/dev/fd0a", 'A',  HD312 } };
-#define INIT_NOOP
+  {"/dev/fd0a", 'A',  MHD312 } };
 #endif
 #endif
 
@@ -823,12 +832,11 @@ struct device devices[] = {
 #define predefined_devices
 #ifdef CPU_m68k
 struct device devices[] = {
-  {"/dev/rfd0b", 'A', ED312 } };
+  {"/dev/rfd0b", 'A', MED312 } };
 #else
 struct device devices[] = {
-  {"/dev/rfd0b", 'A', HD312 } };
+  {"/dev/rfd0b", 'A', MHD312 } };
 #endif
-#define INIT_NOOP
 #endif
 
 
@@ -836,21 +844,20 @@ struct device devices[] = {
 #ifdef __uxp__
 #define predefined_devices
 struct device devices[] = {
-      {"/dev/fpd0",   'A', HD312},
-      {"/dev/fpd0",   'A', DD312},
+      {"/dev/fpd0",   'A', FHD312},
+      {"/dev/fpd0",   'A', FDD312},
 };
 #else
 #define predefined_devices
 struct device devices[] = {
-	{"/dev/rdsk/f1q15dt",	'B', HD514},
-	{"/dev/rdsk/f1d9dt",	'B', DD514},
-	{"/dev/rdsk/f1d8dt",	'B', DDsmall},
-	{"/dev/rdsk/f03ht",	'A', HD312},
-	{"/dev/rdsk/f03dt",	'A', DD312},
+	{"/dev/rdsk/f1q15dt",	'B', FHD514},
+	{"/dev/rdsk/f1d9dt",	'B', FDD514},
+	{"/dev/rdsk/f1d8dt",	'B', FDDsmall},
+	{"/dev/rdsk/f03ht",	'A', FHD312},
+	{"/dev/rdsk/f03dt",	'A', FDD312},
 	{"/dev/rdsk/dos",	'C', GENHD},
 };
 #endif
-#define INIT_NOOP
 #endif /* sysv4 */
 
 #ifdef INIT_GENERIC
@@ -871,6 +878,7 @@ struct device devices[] = {
 #define SET_SSIZE(x,y) return -1
 #endif
 
+#undef INIT_NOOP
 int init_geom(int fd, struct device *dev, struct device *orig_dev,
 	      struct stat *stat)
 {
@@ -982,7 +990,7 @@ int init_geom(int fd, struct device *dev, struct device *orig_dev,
 
 #ifdef INIT_NOOP
 int init_geom(int fd, struct device *dev, struct device *orig_dev,
-	      struct stat *stat)
+			  struct stat *stat)
 {
 	return compare_geom(dev, orig_dev);
 }
@@ -991,13 +999,8 @@ int init_geom(int fd, struct device *dev, struct device *orig_dev,
 #ifdef predefined_devices
 const int nr_const_devices = sizeof(const_devices) / sizeof(*const_devices);
 #else
-int init_geom(int fd, struct device *dev, struct device *orig_dev,
-	      struct stat *stat)
-{
-  return 0;
-}
 struct device devices[]={
-	{"/dev/fd0", 'A', 0, O_EXCL, 0,0, 0,DEF_ARG},
+	{"/dev/fd0", 'A', 0, O_EXCL, 0,0, 0,0, DEF_ARG},
 	/* to shut up Ultrix's native compiler, we can't make this empty :( */
 };
 const nr_const_devices = 0;
