@@ -1,4 +1,4 @@
-#ifdef linux
+#ifdef OS_linux
 
 #include <sys/types.h>
 #include <linux/fd.h>
