@@ -114,7 +114,8 @@ static Class_t FilterClass = {
 	0, /* flush */
 	free_filter,
 	0, /* set geometry */
-	get_data_pass_through
+	get_data_pass_through,
+	0
 };
 
 Stream_t *open_filter(Stream_t *Next)
