@@ -20,21 +20,19 @@ extern int errno;
 
 unsigned int num_clus;			/* total number of cluster */
 
-#ifndef OS_linux
-#define BOOTSIZE 512
-#else
-#define BOOTSIZE 256
-#endif
-
 
 /*
  * Read the boot sector.  We glean the disk parameters from this sector.
  */
-static int read_boot(Stream_t *Stream, struct bootsector * boot)
+static int read_boot(Stream_t *Stream, struct bootsector * boot, int size)
 {	
 	/* read the first sector, or part of it */
-	if (force_read(Stream, (char *) boot, 0, BOOTSIZE) !=
-	    BOOTSIZE)
+	if(!size)
+		size = BOOTSIZE;
+	if(size > 1024)
+		size = 1024;
+
+	if (force_read(Stream, (char *) boot, 0, size) != size)
 		return -1;
 	return 0;
 }
@@ -51,9 +49,10 @@ Class_t FsClass = {
 	read_pass_through, /* read */
 	write_pass_through, /* write */
 	fs_flush, 
-	0, /* free */
+	fs_free, /* free */
 	0, /* set geometry */
-	get_data_pass_through
+	get_data_pass_through,
+	0 /* pre allocate */
 };
 
 static int get_media_type(Stream_t *St, struct bootsector *boot)
@@ -89,6 +88,7 @@ Stream_t *find_device(char drive, int mode, struct device *out_dev,
 	char errmsg[80];
 	Stream_t *Stream;
 	struct device *dev;
+	int r;
 
 	Stream = NULL;
 	sprintf(errmsg, "Drive '%c:' not supported", drive);	
@@ -112,7 +112,7 @@ Stream_t *find_device(char drive, int mode, struct device *out_dev,
 
 		if (!Stream)
 			Stream = SimpleFileOpen(out_dev, dev, name, mode,
-						errmsg, 0);
+						errmsg, 0, 1);
 		else
 			out_dev->use_2m = 0x7f;
 
@@ -120,7 +120,7 @@ Stream_t *find_device(char drive, int mode, struct device *out_dev,
 			continue;
 
 		/* read the boot sector */
-		if (read_boot(Stream, boot)){
+		if ((r=read_boot(Stream, boot, out_dev->blocksize)) < 0){
 			sprintf(errmsg,
 				"init %c: could not read boot sector",
 				drive);
@@ -128,15 +128,26 @@ Stream_t *find_device(char drive, int mode, struct device *out_dev,
 		}
 
 		if((*media= get_media_type(Stream, boot)) <= 0xf0 ){
-			sprintf(errmsg,"init %c: unknown media type", drive);
+			if (boot->jump[2]=='L') 
+				sprintf(errmsg,
+					"diskette %c: is Linux LILO, not DOS", 
+					drive);
+			else 
+				sprintf(errmsg,"init %c: non DOS media", drive);
 			continue;
 		}
 
 		/* set new parameters, if needed */
 		errno = 0;
 		if(SET_GEOM(Stream, out_dev, dev, *media, boot)){
-			sprintf(errmsg, "Can't set disk parameters for %c: %s", 
-				drive, strerror(errno));
+			if(errno)
+				sprintf(errmsg, 
+					"Can't set disk parameters for %c: %s", 
+					drive, strerror(errno));
+			else
+				sprintf(errmsg, 
+					"Can't set disk parameters for %c", 
+					drive);
 			continue;
 		}
 		break;
@@ -154,6 +165,7 @@ Stream_t *find_device(char drive, int mode, struct device *out_dev,
 
 Stream_t *fs_init(char drive, int mode)
 {
+	int blocksize;
 	int media,i;
 	int nhs;
 	int disk_size = 0;	/* In case we don't happen to set this below */
@@ -175,7 +187,9 @@ Stream_t *fs_init(char drive, int mode)
 	This->refs = 1;
 	This->Buffer = 0;
 	This->Class = &FsClass;
+	This->preallocatedClusters = 0;
 	This->drive = drive;
+	This->last = 0;
 
 	This->Direct = find_device(drive, mode, &dev, &boot0, name, &media);
 	if(!This->Direct)
@@ -186,6 +200,19 @@ Stream_t *fs_init(char drive, int mode)
 		fprintf(stderr,"init %c: sector size too big\n", drive);
 		return NULL;
 	}
+	for(i=0; i<16; i++) {
+		if(1 << i == This->sector_size)
+			break;
+	}
+	if(i == 16) {
+		fprintf(stderr, 
+			"init %c: sector size (%d) not a small power of two\n",
+			drive, This->sector_size);
+		return NULL;
+	}
+	This->sectorShift = i;
+	This->sectorMask = This->sector_size - 1;
+
 
 	cylinder_size = dev.heads * dev.sectors;
 	This->serialized = 0;
@@ -198,6 +225,8 @@ Stream_t *fs_init(char drive, int mode)
 		This->dir_len = old_dos[i].dir_len;
 		This->num_fat = 2;
 		This->sector_size = 512;
+		This->sectorShift = 9;
+		This->sectorMask = 511;
 		This->fat_bits = 12;
 		nhs = 0;
 	} else {
@@ -222,8 +251,7 @@ Stream_t *fs_init(char drive, int mode)
 		This->fat_len = WORD(fatlen);
 		This->dir_len = WORD(dirents) * MDIR_SIZE / This->sector_size;
 		This->num_fat = boot0.nfat;
-	}	
-	cylinder_size = dev.sectors * dev.heads;
+	}
 
 	if(!mtools_skip_check && (tot_sectors % dev.sectors)){
 		fprintf(stderr,
@@ -260,11 +288,15 @@ Stream_t *fs_init(char drive, int mode)
 	if (disk_size % 2)
 		disk_size *= 2;
 
+	if(!dev.blocksize || dev.blocksize < This->sector_size)
+		blocksize = This->sector_size;
+	else
+		blocksize = dev.blocksize;
 	if (disk_size)
 		This->Next = buf_init(This->Direct,
-				      disk_size * This->sector_size,
-				      This->sector_size,
-				      disk_size * This->sector_size);
+				      8 * disk_size * blocksize,
+				      disk_size * blocksize,
+				      This->sector_size);
 	else
 		This->Next = This->Direct;
 
@@ -282,3 +314,22 @@ Stream_t *fs_init(char drive, int mode)
 	}
 	return (Stream_t *) This;
 }
+
+char getDrive(Stream_t *Stream)
+{
+	DeclareThis(Fs_t);
+
+	if(This->Class != &FsClass)
+		return getDrive(GetFs(Stream));
+	else
+		return This->drive;
+}
+
+int fsPreallocate(Fs_t *Fs, int size)
+{
+	if(size > 0 && getfreeMin((Stream_t *)Fs, size) != 1)
+		return -1;
+
+	Fs->preallocatedClusters += size;
+	return 0;
+}
