@@ -15,9 +15,6 @@
 #include "xdf_io.h"
 #endif
 
-
-#include <math.h>
-
 #ifndef abs
 #define abs(x) ((x)>0?(x):-(x))
 #endif
@@ -109,7 +106,7 @@ static void init_geometry_boot(struct bootsector *boot, struct device *dev,
 			boot->jump[0] = 0xeb;
 			boot->jump[1] = 0;
 			boot->jump[2] = 0x90;
-			strncpy(boot->banner, "MTOOLS38", 8);
+			strncpy(boot->banner, "MTOOLS39", 8);
 			/* It looks like some versions of DOS are
 			 * rather picky about this, and assume default
 			 * parameters without this, ignoring any
@@ -127,16 +124,24 @@ static int comp_fat_bits(Fs_t *Fs, int estimate,
 	int needed_fat_bits;
 
 	needed_fat_bits = 12;
-	if(tot_sectors > MAX_FAT12_SIZE(Fs->sector_size))
-		needed_fat_bits = 16;       	
-	if(fat32 || tot_sectors > MAX_FAT16_SIZE(Fs->sector_size))
-		needed_fat_bits = 32;		
+
+#define MAX_DISK_SIZE(bits,clusters) \
+	TOTAL_DISK_SIZE((bits), Fs->sector_size, (clusters), \
+			Fs->num_fat, MAX_SECT_PER_CLUSTER)
+
+	if(tot_sectors > MAX_DISK_SIZE(12, FAT12))
+		needed_fat_bits = 16;
+	if(fat32 || tot_sectors > MAX_DISK_SIZE(16, FAT16))
+		needed_fat_bits = 32;
+
+#undef MAX_DISK_SIZE
+
 	if(abs(estimate) && abs(estimate) < needed_fat_bits) {
 		if(fat32) {
 			fprintf(stderr,
 				"Contradiction between FAT size on command line and FAT size in conf file\n");
 			exit(1);
-		}		
+		}
 		fprintf(stderr,
 			"Device too big for a %d bit FAT\n",
 			estimate);
@@ -144,15 +149,19 @@ static int comp_fat_bits(Fs_t *Fs, int estimate,
 	}
 
 	if(needed_fat_bits == 32 && !fat32 && abs(estimate) !=32){
-		fprintf(stderr,"Warning: Using 32 bit FAT.  Drive will only be accessibly by Win95 OEM\n");		
+		fprintf(stderr,"Warning: Using 32 bit FAT.  Drive will only be accessibly by Win95 OEM\n");
 	}
 
 	if(!estimate) {
+		int min_fat16_size;
+
 		if(needed_fat_bits > 12)
 			return needed_fat_bits;
-		else if(tot_sectors < MIN_FAT16_SIZE(Fs->sector_size))
+		min_fat16_size = DISK_SIZE(16, Fs->sector_size, FAT12+1,
+					   Fs->num_fat, 1);
+		if(tot_sectors < min_fat16_size)
 			return 12;
-		else if(tot_sectors >= 2* MIN_FAT16_SIZE(Fs->sector_size))
+		else if(tot_sectors >= 2* min_fat16_size)
 			return 16; /* heuristics */
 	}
 
@@ -165,26 +174,33 @@ static void calc_fat_bits2(Fs_t *Fs, unsigned int tot_sectors, int fat_bits)
 
 	/*
 	 * the "remaining sectors" after directory and boot
-	 * has been accounted for.
+	 * hasve been accounted for.
 	 */
 	rem_sect = tot_sectors - Fs->dir_len - Fs->fat_start;
 	switch(abs(fat_bits)) {
 		case 0:
-			if((rem_sect - 32)/Fs->cluster_size > FAT12)
-				/* big enough for FAT16:
-				 * sectors minus 2 fats of 32
-				 * bits */
+
+#define MY_DISK_SIZE(bits,clusters) \
+			DISK_SIZE( (bits), Fs->sector_size, (clusters), \
+				   Fs->num_fat, Fs->cluster_size)
+
+			if(rem_sect >= MY_DISK_SIZE(16, FAT12 + 1))
+				/* big enough for FAT16 */
 				set_fat16(Fs);
-			else if((rem_sect - 24)/Fs->cluster_size <= FAT12)
-				set_fat12(Fs);
+			else if(rem_sect <= MY_DISK_SIZE(12, FAT12))
+				 /* small enough for FAT12 */
+				 set_fat12(Fs);
 			else {
 				/* "between two chairs",
 				 * augment cluster size, and
 				 * settle it */
-				Fs->cluster_size <<= 1;
+				if(Fs->cluster_size < MAX_SECT_PER_CLUSTER)
+					Fs->cluster_size <<= 1;
 				set_fat12(Fs);
 			}
 			break;
+#undef MY_DISK_SIZE
+
 		case 12:
 			set_fat12(Fs);
 			break;
@@ -210,7 +226,7 @@ static inline void format_root(Fs_t *Fs, char *label, struct bootsector *boot)
 	ch.ignore_entry = -2;
 
 	buf = safe_malloc(Fs->sector_size);
-	RootDir = open_root((Stream_t *)Fs);
+	RootDir = OpenRoot((Stream_t *)Fs);
 	if(!RootDir){
 		fprintf(stderr,"Could not open root directory\n");
 		exit(1);
@@ -218,11 +234,12 @@ static inline void format_root(Fs_t *Fs, char *label, struct bootsector *boot)
 
 	memset(buf, '\0', Fs->sector_size);
 
-	if(Fs->fat_bits == 32)
+	if(Fs->fat_bits == 32) {
 		/* on a FAT32 system, we only write one sector,
 		 * as the directory can be extended at will...*/
 		dirlen = 1;
-	else
+		fatAllocate(Fs, Fs->rootCluster, Fs->end_fat);
+	} else
 		dirlen = Fs->dir_len; 
 	for (i = 0; i < dirlen; i++)
 		WRITES(RootDir, buf, i * Fs->sector_size,  Fs->sector_size);
@@ -236,6 +253,7 @@ static inline void format_root(Fs_t *Fs, char *label, struct bootsector *boot)
 		set_word(boot->dirents, 0);
 	else
 		set_word(boot->dirents, Fs->dir_len * (Fs->sector_size / 32));
+	free(buf);
 }
 
 
@@ -323,6 +341,12 @@ static void calc_fat_size(Fs_t *Fs, unsigned int tot_sectors)
 			" or .mtoolsrc file\n");
 		exit(1);
 	}
+	if ( Fs->num_clus <= FAT12 && Fs->fat_bits > 12 ){
+		fprintf(stderr,"Too few clusters for this fat size."
+			" Please choose a 12-bit fat in your /etc/mtools"
+			" or .mtoolsrc file\n");
+		exit(1);
+	}
 }
 
 
@@ -339,7 +363,7 @@ static inline void inst_boot_prg(struct bootsector *boot)
 	memcpy((char *) boot->jump + offset, 
 	       (char *) bootprog, sizeof(bootprog) /sizeof(bootprog[0]));
 	boot->jump[0] = 0xeb;
-	boot->jump[1] = offset + 1;
+	boot->jump[1] = offset - 1;
 	boot->jump[2] = 0x90;
 	set_word(boot->jump + offset + 20, offset + 24);
 }
@@ -348,20 +372,31 @@ static void calc_cluster_size(struct Fs_t *Fs, unsigned int tot_sectors,
 			      int fat_bits)
 			      
 {
-	unsigned int term, mult, rem_sect;
+	unsigned int max_clusters; /* maximal possible number of sectors for
+				   * this FAT entry length (12/16/32) */
+	unsigned int max_fat_size; /* maximal size of the FAT for this FAT
+				    * entry length (12/16/32) */
+	unsigned int rem_sect; /* remaining sectors after we accounted for
+				* the root directory and boot sector(s) */
 
 	switch(abs(fat_bits)) {
 		case 12:			
-			mult = FAT12;
-			term = MAX_FAT12_FATLEN(Fs->sector_size);
+			max_clusters = FAT12;
+			max_fat_size = Fs->num_fat * 
+				FAT_SIZE(12, Fs->sector_size, max_clusters);
 			break;
 		case 16:
 		case 0: /* still hesititating between 12 and 16 */
-			mult = FAT16;
-			term = MAX_FAT16_FATLEN(Fs->sector_size);
+			max_clusters = FAT16;
+			max_fat_size = Fs->num_fat * 
+				FAT_SIZE(16, Fs->sector_size, max_clusters);
 			break;
-		case 32:
-			Fs->cluster_size = 1;
+		case 32:		  
+			Fs->cluster_size = 8;
+			/* According to
+			 * http://www.microsoft.com/kb/articles/q154/9/97.htm,
+			 * Micro$oft does not support FAT32 with less than 4K
+			 */
 			return;
 		default:
 			fprintf(stderr,"Bad fat size\n");
@@ -369,7 +404,10 @@ static void calc_cluster_size(struct Fs_t *Fs, unsigned int tot_sectors,
 	}
 
 	rem_sect = tot_sectors - Fs->dir_len - Fs->fat_start;
-	while(Fs->cluster_size * mult  + term < rem_sect) {
+
+	/* double the cluster size until we can fill up the disk with
+	 * the maximal number of sectors of this size */
+	while(Fs->cluster_size * max_clusters  + max_fat_size < rem_sect) {
 		if(Fs->cluster_size > 64) {
 			/* bigger than 64. Should fit */
 			fprintf(stderr,
@@ -458,9 +496,11 @@ static void calc_fs_parameters_32(unsigned int tot_sectors,
 	else
 		boot->descr = 0xf0;
 	if(!Fs->cluster_size)
-		/* FAT32 disks have a cluster size of 1 by default.
-		 * After all, we can afford it, can't we? */		
-		Fs->cluster_size = 1;
+		/* According to
+		 * http://www.microsoft.com/kb/articles/q154/9/97.htm,
+		 * Micro$oft does not support FAT32 with less than 4K
+		 */
+		Fs->cluster_size = 8;
 	
 	Fs->dir_len = 0;
 	Fs->num_clus = tot_sectors / Fs->cluster_size;
@@ -511,6 +551,7 @@ void mformat(int argc, char **argv, int dummy)
 	struct device used_dev;
 	int argtracks, argheads, argsectors;
 	int tot_sectors;
+	int blocksize;
 
 	char drive, name[EXPAND_BUF];
 
@@ -644,6 +685,7 @@ void mformat(int argc, char **argv, int dummy)
 	/* check out a drive whose letter and parameters match */	
 	sprintf(errmsg, "Drive '%c:' not supported", drive);	
 	Fs.Direct = NULL;
+	blocksize = 0;
 	for(dev=devices;dev->drive;dev++) {
 		FREE(&(Fs.Direct));
 		/* drive letter */
@@ -669,10 +711,10 @@ void mformat(int argc, char **argv, int dummy)
 #endif
 			Fs.Direct = SimpleFileOpen(&used_dev, dev, name,
 						   O_RDWR | create,
-						   errmsg, 0);
+						   errmsg, 0, 1);
 #ifdef USE_XDF
 		else {
-			used_dev.use_xdf = 1;
+			used_dev.misc_flags |= USE_XDF_FLAG;
 			Fs.Direct = XdfOpen(&used_dev, name, O_RDWR,
 					    errmsg, &info);
 			if(Fs.Direct && !Fs.fat_len)
@@ -704,10 +746,24 @@ void mformat(int argc, char **argv, int dummy)
 		}
 #endif
 		Fs.sector_size = 512;
-		if( !(used_dev.use_2m & 0x7f))
+		Fs.sectorShift = 9;
+		Fs.sectorMask = 511;
+		if( !(used_dev.use_2m & 0x7f)) {
 			Fs.sector_size = 128 << (used_dev.ssize & 0x7f);
+			Fs.sectorShift  = 7 + (used_dev.ssize & 0x7f);
+			Fs.sectorMask  = Fs.sector_size - 1;
+		}
 		SET_INT(Fs.sector_size, msize);
 
+
+		if(!used_dev.blocksize || used_dev.blocksize < Fs.sector_size)
+			blocksize = Fs.sector_size;
+		else
+			blocksize = used_dev.blocksize;
+		
+		if(blocksize > MAX_SECTOR)
+			blocksize = MAX_SECTOR;
+
 		/* do a "test" read */
 		if (!create &&
 		    READS(Fs.Direct, (char *) buf, 0, Fs.sector_size) != 
@@ -738,7 +794,7 @@ void mformat(int argc, char **argv, int dummy)
 			perror("open boot sector");
 			exit(1);
 		}
-		read(fd, buf, Fs.sector_size);
+		read(fd, buf, blocksize);
 		keepBoot = 1;
 	}
 	if(!keepBoot)
@@ -746,9 +802,10 @@ void mformat(int argc, char **argv, int dummy)
 	set_dword(boot->nhs, used_dev.hidden);
 
 	Fs.Next = buf_init(Fs.Direct,
-			   Fs.sector_size * used_dev.heads * used_dev.sectors,
-			   Fs.sector_size,
-			   Fs.sector_size * used_dev.heads * used_dev.sectors);
+			   blocksize * used_dev.heads * used_dev.sectors,
+			   blocksize * used_dev.heads * used_dev.sectors,
+			   blocksize);
+	Fs.Buffer = 0;
 
 	boot->nfat = Fs.num_fat = 2;
 	if(!keepBoot)
@@ -766,7 +823,7 @@ void mformat(int argc, char **argv, int dummy)
 	if(dev->fat_bits == 32) {
 		Fs.primaryFat = 0;
 		Fs.writeAllFats = 1;
-		Fs.fat_start = 3;
+		Fs.fat_start = 32;
 		calc_fs_parameters_32(tot_sectors, &Fs, boot);
 
 		Fs.clus_start = Fs.num_fat * Fs.fat_len + Fs.fat_start;
@@ -781,11 +838,13 @@ void mformat(int argc, char **argv, int dummy)
 		set_dword(boot->ext.fat32.rootCluster, Fs.rootCluster = 2);
 
 		/* info sector */
-		set_word(boot->ext.fat32.infoSector, Fs.infoSectorLoc = 2);
+		set_word(boot->ext.fat32.infoSector, Fs.infoSectorLoc = 1);
+		Fs.infoSectorLoc = 1;
 
 		/* no backup boot sector */
-		set_word(boot->ext.fat32.backupBoot, 1);
+		set_word(boot->ext.fat32.backupBoot, 6);
 	} else {
+		Fs.infoSectorLoc = 0;
 		Fs.fat_start = 1;
 		calc_fs_parameters(&used_dev, tot_sectors, &Fs, boot);
 		boot->ext.old.physdrive = 0x00;
@@ -830,28 +889,18 @@ void mformat(int argc, char **argv, int dummy)
 		inst_boot_prg(boot);
 	if(dev->use_2m & 0x7f)
 		Fs.num_fat = 1;
+	Fs.lastFatSectorNr = 0;
+	Fs.lastFatSectorData = 0;
 	zero_fat(&Fs, boot->descr);
-	if(Fs.fat_bits == 32) {
-		/* initialize info sector */
-		Fs.infoSector = (InfoSector_t *) safe_malloc(Fs.sector_size);
-		set_dword(Fs.infoSector->signature, INFOSECT_SIGNATURE);
-		set_dword(Fs.infoSector->count, Fs.num_clus);
-		set_dword(Fs.infoSector->pos, 2);
-
-		/* this will trigger writing it */
-		Fs.fat_dirty = 1;
-
-		/* root directory: allocate one sector */
-		Fs.fat_encode(&Fs, Fs.rootCluster, Fs.end_fat);
-	} else
-		Fs.infoSector = 0;
+	Fs.freeSpace = Fs.num_clus;
+	Fs.last = 2;
 
 #ifdef USE_XDF
 	if(format_xdf)
 		for(i=0; 
 		    i < (info.BadSectors+Fs.cluster_size-1)/Fs.cluster_size; 
 		    i++)
-			Fs.fat_encode(&Fs, i+2, 0xfff7);
+			fatEncode(&Fs, i+2, 0xfff7);
 #endif
 
 	format_root(&Fs, label, boot);
@@ -863,6 +912,8 @@ void mformat(int argc, char **argv, int dummy)
 	}
 	FLUSH((Stream_t *)&Fs); /* flushes Fs. 
 				 * This triggers the writing of the FAT */
+	FREE(&Fs.Next);
+	Fs.Class->freeFunc((Stream_t *)&Fs);
 #ifdef USE_XDF
 	if(format_xdf && isatty(0) && !getenv("MTOOLS_USE_XDF"))
 		fprintf(stderr,
