@@ -14,8 +14,6 @@
 #include "scsi.h"
 #include "partition.h"
 
-#include <math.h>
-
 #define tolinear(x) \
 (sector(x)-1+(head(x)+cyl(x)*used_dev->heads)*used_dev->sectors)
 
@@ -38,6 +36,7 @@ static void set_offset(hsc *h, int offset, int heads, int sectors)
 
 		head = offset % heads;
 		cyl = offset / heads;
+		if(cyl > 1023) cyl = 1023;
 	}
 
 	h->head = head;
@@ -86,6 +85,7 @@ int consistencyCheck(struct partition *partTable, int doprint, int verbose,
 		*j = i;
 
 		if(used_dev &&
+		   cyl(partTable[i].start) != 1023 &&
 		   tolinear(partTable[i].start) != BEGIN(partTable[i])) {
 			fprintf(stderr,
 				"Start position mismatch for partition %d\n",
@@ -93,6 +93,7 @@ int consistencyCheck(struct partition *partTable, int doprint, int verbose,
 			inconsistency=1;
 		}
 		if(used_dev &&
+		   cyl(partTable[i].end) != 1023 &&
 		   tolinear(partTable[i].end)+1 != END(partTable[i])) {
 			fprintf(stderr,
 				"End position mismatch for partition %d\n",
@@ -380,7 +381,7 @@ void mpartition(int argc, char **argv, int dummy)
 		expand(dev->name, name);
 		Stream = SimpleFileOpen(&used_dev, dev, name,
 					dirty ? O_RDWR : O_RDONLY, 
-					errmsg, open2flags);
+					errmsg, open2flags, 1);
 
 		if (!Stream) {
 			sprintf(errmsg,"init: open: %s", strerror(errno));
@@ -391,7 +392,7 @@ void mpartition(int argc, char **argv, int dummy)
 		/* try to find out the size */
 		if(!sizetest)
 			tot_sectors = 0;
-		if(dev->scsi) {
+		if(IS_SCSI(dev)) {
 			unsigned char cmd[10];
 			unsigned char data[10];
 			cmd[0] = SCSI_READ_CAPACITY;
@@ -417,7 +418,7 @@ void mpartition(int argc, char **argv, int dummy)
 			continue;
 		}
 		if(verbose>=2)
-			print_sector("Read sector", buf);
+			print_sector("Read sector", buf, 512);
 		break;
 	}
 
@@ -637,13 +638,13 @@ void mpartition(int argc, char **argv, int dummy)
 	if(dirty) {
 		/* write data back to the disk */
 		if(verbose>=2)
-			print_sector("Writing sector", buf);
+			print_sector("Writing sector", buf, 512);
 		if (WRITES(Stream, (char *) buf, 0, 512) != 512) {
 			fprintf(stderr,"Error writing partition table");
 			exit(1);
 		}
 		if(verbose>=3)
-			print_sector("Sector written", buf);
+			print_sector("Sector written", buf, 512);
 		FREE(&Stream);
 	}
 	exit(0);
