@@ -21,6 +21,7 @@ static const struct dispatch {
 	{"mdel",mdel, 0},
 	{"mdeltree",mdel, 2},
 	{"mdir",mdir, 0},
+	{"mdu",mdu, 0},
 	{"mformat",mformat, 0},
 	{"minfo", minfo, 0},
 	{"mlabel",mlabel, 0},
@@ -31,6 +32,7 @@ static const struct dispatch {
 	{"mread",mcopy, 0},
 	{"mmove",mmove, 0},
 	{"mren",mmove, 1},
+	{"mshowfat", mshowfat, 0},
 	{"mtoolstest", mtoolstest, 0},
 	{"mtype",mcopy, 1},
 	{"mwrite",mcopy, 0},
@@ -38,9 +40,9 @@ static const struct dispatch {
 };
 #define NDISPATCH (sizeof dispatch / sizeof dispatch[0])
 
-void main(int argc,char **argv)
+int main(int argc,char **argv)
 {
-	char *name;
+	const char *name;
 	int i;
 
 	init_privs();
@@ -68,7 +70,7 @@ void main(int argc,char **argv)
 		}
 		sprintf(command, "a.out %d", euid);
 		system(command);
-		exit(1);
+		return 1;
 	}
 #endif
 
@@ -102,7 +104,7 @@ void main(int argc,char **argv)
 		printf("disable-raw-term ");
 #endif
 		printf("\n");
-		exit(0);
+		return 0;
 	}
 
 	/* check whether the compiler lays out structures in a sane way */
@@ -111,13 +113,10 @@ void main(int argc,char **argv)
 	   sizeof(struct vfat_subentry) !=32) {
 		fprintf(stderr,"Mtools has not been correctly compiled\n");
 		fprintf(stderr,"Recompile it using a more recent compiler\n");
-		exit(137);
+		return 137;
 	}
 
-	if ((name = strrchr(argv[0],'/')))
-		name++;
-	else
-		name = argv[0];
+	name = _basename(argv[0]);
 	progname = argv[0];
 
 	/* this allows the different tools to be called as "mtools -c <command>"
@@ -133,8 +132,6 @@ void main(int argc,char **argv)
 		name = argv[0];
 	}
 
-	argv[0] = name;
-	
 	read_config();
 	setup_signal();
 	for (i = 0; i < NDISPATCH; i++) {
@@ -151,5 +148,5 @@ void main(int argc,char **argv)
 	}
 	putc('\n', stderr);
 
-	exit(1);
+	return 1;
 }
