@@ -11,6 +11,11 @@
 /*  (c) 1996   Grant R. Guenther,  based on work of Itai Nahshon  */
 /*  http://www.torque.net/ziptool.html  */
 
+
+/* Unprotect-till-eject modes and mount tests added
+ * by Ilya Ovchinnikov <ilya@socio.msu.su>
+ */
+
 #include "sysincludes.h"
 #include "mtools.h"
 #include "scsi.h"
@@ -21,28 +26,112 @@
 
 #ifdef OS_linux
 #include <linux/fs.h>
+#include <linux/major.h>
 #endif
 
+int test_mounted ( char *dev )
+{
+#ifdef HAVE_MNTENT_H
+	struct mntent	*mnt;
+	struct stat	st_dev, st_mnt;
+	FILE		*mtab;
+/*
+ * Now check if any partition of this device is already mounted (this
+ * includes checking if the device is mounted under a different name).
+ */
+	
+	if (stat (dev, &st_dev)) {
+		fprintf (stderr, "%s: stat(%s) failed: %s.\n",
+			 progname, dev, strerror (errno));
+		exit(1);
+	}
+	
+	if (!S_ISBLK (st_dev.st_mode)) /* not a block device, cannot 
+					* be mounted */
+		return 0;
+	
+	if ((mtab = setmntent (MOUNTED, "r")) == NULL) {
+		fprintf (stderr, "%s: can't open %s.\n",
+			 progname, MOUNTED);
+		exit(1);
+	}
+	
+	while ( ( mnt = getmntent (mtab) ) ) {
+		if (0
+#ifdef MNTTYPE_NFS
+		    || !strcmp (mnt->mnt_type, MNTTYPE_NFS)
+#endif
+#ifdef MNTTYPE_PROC
+		    ||  !strcmp (mnt->mnt_type, MNTTYPE_PROC)
+#endif
+#ifdef MNTTYPE_SMBFS
+		    ||  !strcmp (mnt->mnt_type, MNTTYPE_SMBFS)
+#endif
+#ifdef MNTTYPE_IGNORE
+		    ||  !strcmp (mnt->mnt_type, MNTTYPE_IGNORE)
+#endif
+			)
+			continue;
+	    
+		if (stat (mnt->mnt_fsname, &st_mnt)) {
+			fprintf (stderr, "%s: stat(%s) failed: %s.\n",
+				 progname, mnt->mnt_fsname, strerror (errno));
+			endmntent (mtab);
+			exit(1);
+		}
+		
+		if (S_ISBLK (st_mnt.st_mode)) {
+#ifdef OS_linux
+			if (MAJOR(st_mnt.st_rdev) == SCSI_DISK_MAJOR &&
+			    MINOR(st_mnt.st_rdev) >= MINOR(st_dev.st_rdev) &&
+			    MINOR(st_mnt.st_rdev) <= MINOR(st_dev.st_rdev)+15) {
+				fprintf (stderr, 
+					 "Device %s%d is mounted on %s.\n", 
+					 dev, MINOR(st_mnt.st_rdev) - MINOR(st_dev.st_rdev),
+					 mnt->mnt_dir);
+#else
+			if(st_mnt.st_rdev != st_dev.st_mode) {
+#endif
+				endmntent (mtab);
+				return 1;
+			}
+		}
+	}
+	endmntent (mtab);
+#endif
+	return 0;
+}
+
+
 static void usage(void)
 {
 	fprintf(stderr, 
 		"Mtools version %s, dated %s\n", 
 		mversion, mdate);
 	fprintf(stderr, 
-		"Usage: %s [-V] [-q] [-e] [-r|-w|-p|-x] [drive:]\n"
+		"Usage: %s [-V] [-q] [-e] [-u] [-r|-w|-p|-x] [drive:]\n"
 		"\t-q print status\n"
 		"\t-e eject disk\n"
 		"\t-f eject disk even when mounted\n"
 		"\t-r write protected (read-only)\n"
 		"\t-w not write-protected (read-write)\n"
 		"\t-p password write protected\n"
-		"\t-x password protected\n", 
+		"\t-x password protected\n"
+		"\t-u unprotect till disk ejecting\n", 
 		progname);
 	exit(1);
 }
 
 
-static int get_zip_status(int fd)
+enum mode_t {
+	ZIP_RW = 0,
+	ZIP_RO = 2,
+	ZIP_RO_PW = 3,
+	ZIP_PW = 5,
+	ZIP_UNLOCK_TIL_EJECT = 8
+};
+
+static enum mode_t get_zip_status(int fd)
 {
 	unsigned char status[128];
 	unsigned char cdb[6] = { 0x06, 0, 0x02, 0, sizeof status, 0 };
@@ -91,37 +180,47 @@ void mzip(int argc, char **argv, int type)
 	enum { ZIP_NIX    =      0,
 	       ZIP_STATUS = 1 << 0,
 	       ZIP_EJECT  = 1 << 1,
-	       ZIP_RW     = 1 << 2,
-	       ZIP_RO     = 1 << 3,
-	       ZIP_RO_PW  = 1 << 4,
-	       ZIP_PW     = 1 << 5,
-	       ZIP_FORCE  = 1 << 6
+	       ZIP_MODE_CHANGE = 1 << 2,
+	       ZIP_FORCE  = 1 << 3
 	} request = ZIP_NIX;
+
+	enum mode_t newMode = ZIP_RW;
+	enum mode_t oldMode = ZIP_RW;
+
+#define setMode(x) \
+	if(request & ZIP_MODE_CHANGE) usage(); \
+	request |= ZIP_MODE_CHANGE; \
+	newMode = x; \
+	break;
 	
 	/* get command line options */
-	while ((c = getopt(argc, argv, "efpqrwx")) != EOF) {
+	while ((c = getopt(argc, argv, "efpqrwxu")) != EOF) {
 		switch (c) {
 			case 'f':
+				if (get_real_uid()) {
+					fprintf(stderr, 
+						"Only root can use force. Sorry.\n");
+					exit(1);
+				}
 				request |= ZIP_FORCE;
 				break;
 			case 'e': /* eject */
 				request |= ZIP_EJECT;
 				break;
-			case 'p': /* password read-only */
-				request |= ZIP_RO_PW;
-				break;
 			case 'q': /* status query */
 				request |= ZIP_STATUS;
 				break;
+
+			case 'p': /* password read-only */
+				setMode(ZIP_RO_PW);
 			case 'r': /* read-only */
-				request |= ZIP_RO;
-				break;
+				setMode(ZIP_RO);
 			case 'w': /* read-write */
-				request |= ZIP_RW;
-				break;
+				setMode(ZIP_RW);
 			case 'x': /* password protected */
-				request |= ZIP_PW;
-				break;
+				setMode(ZIP_PW);
+			case 'u': /* password protected */
+				setMode(ZIP_UNLOCK_TIL_EJECT)
 			default:  /* unrecognized */
 				usage();
 		}
@@ -129,10 +228,6 @@ void mzip(int argc, char **argv, int type)
 	
 	if (request == ZIP_NIX) request = ZIP_STATUS;  /* default action */
 
-	if(!!(request & ZIP_RW) + !!(request & ZIP_RO) +
-	   !!(request & ZIP_RO_PW) + !!(request & ZIP_PW) > 1)
-		usage();
-	
 	if (argc - optind > 1 || 
 	    (argc - optind == 1 &&
 	     (!argv[optind][0] || argv[optind][1] != ':')))
@@ -160,12 +255,19 @@ void mzip(int argc, char **argv, int type)
 		if (dev->drive != drive) 
 			continue;
 		expand(dev->name, name);
+		if ((request & ZIP_MODE_CHANGE) &&
+		    !(request & ZIP_FORCE) &&
+		    test_mounted(name)) {
+			fprintf(stderr, 
+				"Can\'t change status of mounted device\n");
+			exit(1);
+		}
 		precmd(dev);
 
-		if(dev->privileged)
+		if(IS_PRIVILEGED(dev))
 			reclaim_privs();
-		fd = open(name, O_RDONLY | dev->mode); 
-		if(dev->privileged)
+		fd = open(name, O_RDONLY | O_NDELAY /* O_RDONLY  | dev->mode*/);
+		if(IS_PRIVILEGED(dev))
 			drop_privs();
 
 		/* need readonly, else we can't
@@ -175,9 +277,7 @@ void mzip(int argc, char **argv, int type)
 			continue;
 		closeExec(fd);
 
-
-		if (!(request & (ZIP_RW | ZIP_RO | ZIP_RO_PW | 
-				 ZIP_PW | ZIP_STATUS)))
+		if (!(request & (ZIP_MODE_CHANGE | ZIP_STATUS)))
 			/* if no mode change or ZIP specific status is
 			 * involved, the command (eject) is applicable
 			 * on all drives */
@@ -225,35 +325,57 @@ void mzip(int argc, char **argv, int type)
 		exit(1);
 	}
 	
-	if (request & (ZIP_RW | ZIP_RO | ZIP_RO_PW | ZIP_PW)) {
-		int ret;
-		char *passwd, dummy[1];
-		int newmode;
+	if (request & (ZIP_MODE_CHANGE | ZIP_STATUS))
+		oldMode = get_zip_status(fd);
 
-		/* unlock the disk by trying the tools disk password first */
-		if (get_zip_status(fd) & 0x01) {  /* unlock first */
+	if (request & ZIP_MODE_CHANGE) {
+		/* request temp unlock, and disk is already unlocked */
+		if(newMode == ZIP_UNLOCK_TIL_EJECT &&
+		   (oldMode & ZIP_UNLOCK_TIL_EJECT))
+			request &= ~ZIP_MODE_CHANGE;
+		
+		/* no password change requested, and disk is already
+		 * in the requested state */
+		if(!(newMode & 0x01) && newMode == oldMode)
+			request &= ~ZIP_MODE_CHANGE;
+	}
+
+	if (request & ZIP_MODE_CHANGE) {
+		int ret;
+		enum mode_t unlockMode, unlockMask;
+		char *passwd, dummy[1];
+
+		if(newMode == ZIP_UNLOCK_TIL_EJECT) {
+			unlockMode = newMode | oldMode;
+			unlockMask = 9;
+		} else {
+			unlockMode = newMode & ~0x5;
+			unlockMask = 1;
+		}
+
+		if ((oldMode & unlockMask) == 1) {  /* unlock first */
 			char *s, *passwd;
 			passwd = "APlaceForYourStuff";
 			if ((s = strchr(passwd, '\n'))) *s = '\0';  /* chomp */
-			iomega_command(fd, 0, passwd);
+			iomega_command(fd, unlockMode, passwd);
 		}
-
-		if (get_zip_status(fd) & 0x01) {  /* unlock first */
+		
+		if ((get_zip_status(fd) & unlockMask) == 1) { /* unlock first */
 			char *s, *passwd;
 			passwd = getpass("Password: ");
 			if ((s = strchr(passwd, '\n'))) *s = '\0';  /* chomp */
-			if((ret=iomega_command(fd, 0, passwd))){
+			if((ret=iomega_command(fd, unlockMode, passwd))){
 				if (ret == -1) perror("passwd: ");
 				else fprintf(stderr, "wrong password\n");
 				exit(1);
 			}
-			if(get_zip_status(fd) & 0x01) {
+			if((get_zip_status(fd) & unlockMask) == 1) {
 				fprintf(stderr, "wrong password\n");
 				exit(1);
 			}
 		}
 		
-		if (request & (ZIP_RO_PW | ZIP_PW)) {			
+		if (newMode & 0x1) {
 			char first_try[_PASSWORD_LEN];
 			
 			passwd = getpass("Enter new password:");
@@ -268,27 +390,11 @@ void mzip(int argc, char **argv, int type)
 			passwd = dummy;
 			dummy[0] = '\0';
 		}
-			
-		switch (request & 
-			(ZIP_RW | ZIP_RO | ZIP_RO_PW | ZIP_PW)) {
-			case ZIP_RW:    
-				newmode = 0x00;
-				break;
-			case ZIP_RO:   
-				newmode = 0x02;
-				break;
-			case ZIP_RO_PW:
-				newmode = 0x03;
-				break;
-			case ZIP_PW:  
-				newmode = 0x05;
-				break;
-			default:
-				newmode = 0;
-				break;
-		}
-			
-		if((ret=iomega_command(fd, newmode, passwd))){
+
+		if(newMode == ZIP_UNLOCK_TIL_EJECT)
+			newMode |= oldMode;
+
+		if((ret=iomega_command(fd, newMode, passwd))){
 			if (ret == -1) perror("set passwd: ");
 			else fprintf(stderr, "password not changed\n");
 			exit(1);
@@ -301,28 +407,32 @@ void mzip(int argc, char **argv, int type)
 	}
 	
 	if (request & ZIP_STATUS) {
-		int status;
-		
-		switch (status = get_zip_status(fd)) {
-			case 0:  
-				printf("Drive '%c:' is not write-protected\n", 
+		char *unlocked;
+
+		if(oldMode & 8)
+			unlocked = " and unlocked until eject";
+		else
+			unlocked = "";		
+		switch (oldMode & ~8) {
+			case ZIP_RW:  
+				printf("Drive '%c:' is not write-protected\n",
 				       drive);
 				break;
-			case 2:
-				printf("Drive '%c:' is write-protected\n",
-				       drive);
+			case ZIP_RO:
+				printf("Drive '%c:' is write-protected%s\n",
+				       drive, unlocked);
 				break;
-			case 3: 
-				printf("Drive '%c:' is password write-protected\n", 
-				       drive);
+			case ZIP_RO_PW: 
+				printf("Drive '%c:' is password write-protected%s\n", 
+				       drive, unlocked);
 				break;
-			case 5:  
-				printf("Drive '%c:' is password protected\n", 
-				       drive);
+			case ZIP_PW:  
+				printf("Drive '%c:' is password protected%s\n", 
+				       drive, unlocked);
 				break;
 			default: 
 				printf("Unknown protection mode %d of drive '%c:'\n",
-				       status, drive);
+				       oldMode, drive);
 				break;				
 		}		
 	}
