@@ -53,7 +53,7 @@ int lock_dev(int fd, int mode, struct device *dev)
 #endif /* LOCKF */
 #endif /* FLOCK */
 
-	if(dev && dev->nolock)
+	if(IS_NOLOCK(dev))
 		return 0;
 
 #if (defined(HAVE_FLOCK) && defined (LOCK_EX) && defined(LOCK_NB))
@@ -96,6 +96,7 @@ static int file_io(Stream_t *Stream, char *buf, int where, int len,
 	int ret;
 
 	where += This->offset;
+
 	if (This->seekable && where != This->lastwhere ){
 		if(lseek( This->fd, where, SEEK_SET) < 0 ){
 			perror("seek");
@@ -105,7 +106,7 @@ static int file_io(Stream_t *Stream, char *buf, int where, int len,
 	}
 	ret = io(This->fd, buf, len);
 	if ( ret == -1 ){
-		perror("read");
+		perror("plain_io");
 		This->lastwhere = -1;
 		return -1;
 	}
@@ -254,7 +255,7 @@ static int file_data(Stream_t *Stream, long *date, size_t *size,
 #define MAXBLKSPERCMD 255
 
 static void scsi_init(int);
-static scsi_sector_size=512;
+static int scsi_sector_size=512;
 
 static void scsi_init(int fd)
 {
@@ -409,7 +410,8 @@ static Class_t ScsiClass = {
 	file_flush,
 	file_free,
 	file_geom,
-	file_data
+	file_data,
+	0 /* pre-allocate */
 };
 
 
@@ -424,14 +426,15 @@ static Class_t SimpleFileClass = {
 
 
 Stream_t *SimpleFileOpen(struct device *dev, struct device *orig_dev,
-			 const char *name, int mode, char *errmsg, int mode2)
+			 const char *name, int mode, char *errmsg, 
+			 int mode2, int locked)
 {
 	SimpleFile_t *This;
 
 	This = New(SimpleFile_t);
 	if (!This){
-		fprintf(stderr,"Out of memory error\n");
-		exit(1);
+		printOom();
+		return 0;
 	}
 	This->seekable = 1;
 
@@ -451,17 +454,17 @@ Stream_t *SimpleFileOpen(struct device *dev, struct device *orig_dev,
 	
 	if(dev) {
 		if(!(mode2 & NO_PRIV))
-			This->privileged = dev->privileged;
+			This->privileged = IS_PRIVILEGED(dev);
 		mode |= dev->mode;
 	}
 
 	precmd(dev);
-	if(dev && dev->privileged && !(mode2 & NO_PRIV))
+	if(IS_PRIVILEGED(dev) && !(mode2 & NO_PRIV))
 		reclaim_privs();
 
 	This->fd = open(name, mode, 0666);
 
-	if(dev && dev->privileged && !(mode2 & NO_PRIV))
+	if(IS_PRIVILEGED(dev) && !(mode2 & NO_PRIV))
 		drop_privs();
 		
 	if (This->fd < 0) {
@@ -472,7 +475,8 @@ Stream_t *SimpleFileOpen(struct device *dev, struct device *orig_dev,
 		return NULL;
 	}
 
-	closeExec(This->fd);
+	if(IS_PRIVILEGED(dev) && !(mode2 & NO_PRIV))
+		closeExec(This->fd);
 
 	if (fstat(This->fd, &This->stat) < 0){
 		Free(This);
@@ -484,7 +488,7 @@ Stream_t *SimpleFileOpen(struct device *dev, struct device *orig_dev,
 	}
 
 	/* lock the device on writes */
-	if (lock_dev(This->fd, mode == O_RDWR, dev)) {
+	if (locked && lock_dev(This->fd, mode == O_RDWR, dev)) {
 		close(This->fd);
 		Free(This);
 		if(errmsg)
@@ -495,8 +499,9 @@ Stream_t *SimpleFileOpen(struct device *dev, struct device *orig_dev,
 	}
 		
 	/* set default parameters, if needed */
-	if (dev){
-		if (init_geom(This->fd, dev, orig_dev, &This->stat)){
+	if (dev){		
+		if (IS_MFORMAT_ONLY(dev) &&
+			init_geom(This->fd, dev, orig_dev, &This->stat)){
 			close(This->fd);
 			Free(This);
 			if(errmsg)
@@ -522,9 +527,13 @@ Stream_t *SimpleFileOpen(struct device *dev, struct device *orig_dev,
 	/* AK: introduce new "scsi=1" statement to specifically set
 	 * this option. Indeed, there could conceivably be partitioned
 	 * devices where low level scsi commands will not be needed */
-	if(dev && dev->scsi) {
+	if(IS_SCSI(dev)) {
 		This->Class = &ScsiClass;
+		if(This->privileged)
+			reclaim_privs();
 		scsi_init(This->fd);
+		if(This->privileged)
+			drop_privs();
 	}
 	while(!(mode2 & NO_OFFSET) &&
 	      dev && dev->partition && dev->partition <= 4) {
