@@ -9,7 +9,8 @@
 #define NO_OFFSET 2
 
 Stream_t *SimpleFileOpen(struct device *dev, struct device *orig_dev,
-			 const char *name, int mode, char *errmsg, int mode2);
+			 const char *name, int mode, char *errmsg, int mode2,
+			 int locked);
 int check_parameters(struct device *ref, struct device *testee);
 
 int get_fd(Stream_t *Stream);
