@@ -103,6 +103,11 @@ void destroy_privs(void)
 }
 
 
+uid_t get_real_uid(void)
+{
+	return ruid;
+}
+
 void init_privs(void)
 {
 	euid = geteuid();
@@ -110,12 +115,12 @@ void init_privs(void)
 	egid = getegid();
 	rgid = getgid();
 
-#ifndef FD_CLOEXEC
+#ifndef F_SETFD
 	if(euid != ruid) {
 		fprintf(stderr,
 			"Setuid installation not supported on this platform\n");
 		fprintf(stderr,
-			"Missing FD_CLOEXEC");
+			"Missing F_SETFD");
 		exit(1);
 	}
 #endif
@@ -150,7 +155,7 @@ void init_privs(void)
 
 void closeExec(int fd)
 {
-#ifdef FD_CLOEXEC
-	fcntl(fd, F_SETFL, FD_CLOEXEC);
+#ifdef F_SETFD
+	fcntl(fd, F_SETFD, 1);
 #endif
 }
