@@ -157,6 +157,26 @@ extern int optind, opterr;
 # endif
 #endif
 
+
+#if defined(OS_aux) && !defined(_SYSV_SOURCE)
+/* compiled in POSIX mode, this is left out unless SYSV */
+#define	NCC	8
+struct termio {
+	unsigned short	c_iflag;	/* input modes */
+	unsigned short	c_oflag;	/* output modes */
+	unsigned short	c_cflag;	/* control modes */
+	unsigned short	c_lflag;	/* line discipline modes */
+	char	c_line;			/* line discipline */
+	unsigned char	c_cc[NCC];	/* control chars */
+};
+extern int ioctl(int fildes, int request, void *arg);
+#endif
+
+
+#ifdef HAVE_MNTENT_H
+# include <mntent.h>
+#endif
+
 #ifdef HAVE_SYS_PARAM_H
 # include <sys/param.h>
 #endif
@@ -179,10 +199,7 @@ extern int optind, opterr;
 
 #include <errno.h>
 extern int errno;
-#if !defined OS_netbsd && !defined OS_freebsd && !(defined __GLIBC__ && __GLIBC__ >= 2)
-/* NetBSD seems to choke on this, due to a slightly non-standard definition */
-extern char *sys_errlist[];
-#endif
+
 #include <pwd.h>
 
 
@@ -222,6 +239,15 @@ extern char *sys_errlist[];
 # include <linux/fd.h>
 #endif
 
+#ifdef sgi
+#define MSGIHACK __EXTENSIONS__
+#undef __EXTENSIONS__
+#endif
+#include <math.h>
+#ifdef sgi
+#define __EXTENSIONS__ MSGIHACK
+#undef MSGIHACK
+#endif
 
 /* missing functions */
 #ifndef HAVE_SRANDOM
@@ -305,6 +331,13 @@ int strncasecmp(const char *s1, const char *s2, size_t n);
 char *getpass(const char *prompt);
 #endif
 
+#if 0
+#ifndef HAVE_BASENAME
+const char *basename(const char *filename);
+#endif
+#endif
+
+const char *_basename(const char *filename);
 
 #ifndef __STDC__
 # ifndef signed
@@ -392,4 +425,21 @@ struct utimbuf
 #define S_ISDIR(mode)   (((mode) & (_S_IFMT)) == (_S_IFDIR))
 #endif
 
+
+#if 0
+
+#define malloc(x) mymalloc(x)
+#define calloc(x,y) mycalloc(x,y)
+#define free(x) myfree(x)
+#define realloc(x,y) myrealloc(x,y)
+#define strdup(a) mystrdup(a)
+
+void *mycalloc(size_t nmemb, size_t size);
+void *mymalloc(size_t size);
+void myfree(void *ptr);
+void *myrealloc(void *ptr, size_t size);
+char *mystrdup(char *a);
+
+#endif
+
 #endif
